/*
 * 版权所有 (C) 2015 知启蒙(ZHIQIM) 保留所有权利。[遇见知启蒙，邂逅框架梦]
 * 
 * https://zhiqim.org/project/zhiqim_framework/zhiqim_kernel.htm
 *
 * Zhiqim Kernel is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 */
package org.zhiqim.kernel;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.zhiqim.kernel.annotation.AnGlobal;
import org.zhiqim.kernel.annotation.AnNew;
import org.zhiqim.kernel.util.Annotations;
import org.zhiqim.kernel.util.Classes;
import org.zhiqim.kernel.util.Randoms;
import org.zhiqim.kernel.util.codes.DES;

public class ZhiqimContextLoader extends URLClassLoader
{
    private String name = "ZhiqimContextLoader";
    private Map<String, byte[]> classMap = new ConcurrentHashMap<>();
    private Map<String, URL> resourceMap = new ConcurrentHashMap<>();
    private String id = Randoms.digits(6);

    public ZhiqimContextLoader(URL[] urls, ClassLoader parent)
    {
        super(urls, parent);
    }

    public void addJarFile(String filePath, boolean isEncrypt) throws Exception
    {
        File file = new File(filePath);
        if (file.exists())
        {
            JarFile jar = new JarFile(file);
            loadJar(jar, isEncrypt);
        }
    }

    public void loadJar(JarFile jarFile, boolean isEncrypt) throws Exception
    {
        String key = jarFile.getManifest().getMainAttributes().getValue("DES");
        Enumeration<JarEntry> es = jarFile.entries();
        while (es.hasMoreElements())
        {
            JarEntry jarEntry = (JarEntry) es.nextElement();
            if (jarEntry.isDirectory())
                continue;

            String name = jarEntry.getName();
            if (name == null)
                continue;

            if (name.endsWith(".class"))
            {// 类文件
                InputStream input = null;
                ByteArrayOutputStream output = null;
                try
                {
                    input = jarFile.getInputStream(jarEntry);
                    output = new ByteArrayOutputStream();
                    int bufferSize = 1024;
                    byte[] buffer = new byte[bufferSize];
                    int read = 0;
                    while ((read = input.read(buffer)) != -1)
                    {
                        output.write(buffer, 0, read);
                    }
                    byte[] b = output.toByteArray();
                    String className = name.replace('/', '.').replace(".class", "");
                    if (isEncrypt)
                    {
                        byte[] bytes = DES.decrypt(b, key);
                        classMap.put(className, bytes);
                    }
                    else
                    {
                        classMap.put(className, b);
                    }

                }
                catch (IOException e)
                {}
                finally
                {
                    if (output != null)
                        output.close();
                    if (input != null)
                        input.close();
                }
            }

            if (!name.endsWith(".class") && !name.endsWith(".java"))
            {// 资源文件
                String path = "jar:file:" + jarFile.getName() + "!/" + name;
                URL url = new URL(path);
                resourceMap.put(name, url);
            }
        }
    }

    /**
     * 卸载类及资源
     * @return
     */
    public boolean unload()
    {
        classMap.clear();
        resourceMap.clear();
        return true;
    }

    public void loadAlias()
    {
        for (String className : classMap.keySet())
        {
            setClassAlias(className);
        }
    }
    
    public void removeAlias()
    {
        for (String className : classMap.keySet())
        {
            removeClassAlias(className);
        }
    }

    
    /**
     * 设置别名
     * @param className
     * @return
     */
    private boolean setClassAlias(String className)
    {
        Class<?> cls = Classes.forName(className);
        if (cls == null)
            return true;
        
        if (cls.isAnnotationPresent(AnGlobal.class) && cls.isAnnotationPresent(AnNew.class))
        {
            return false;
        }
        
        String alias = Annotations.getClassAlias(cls);
        if (alias == null)
            return true;
        
        Global.addClass(alias, cls);
        return true;
    }
    
    private boolean removeClassAlias(String className)
    {
        Class<?> cls = Classes.forName(className);
        if (cls == null)
            return true;
        
        if (cls.isAnnotationPresent(AnGlobal.class) && cls.isAnnotationPresent(AnNew.class))
        {
            return false;
        }
        
        String alias = Annotations.getClassAlias(cls);
        if (alias == null)
            return true;
        
        Global.removeClass(alias);
        return true;
    }

    @Override
    protected Class<?> findClass(String name)
    {
        try
        {
            byte[] result = classMap.get(name);
            if (result != null)
                return defineClass(name, result, 0, result.length);
        }
        catch (Exception e)
        {}
        return null;
    }

    @Override
    public URL findResource(String name)
    {
        return resourceMap.get(name);
    }

    public String toString()
    {
        return name + "-" + id;
    }
}
